@@ -5,8 +5,9 @@
 This module contains unit tests for the validator functions in the amd-debug-tools package.
 """
 
-from unittest.mock import patch, mock_open
+from unittest.mock import patch, mock_open, Mock
 
+import os
 import logging
 import unittest
 import math
@@ -70,7 +71,8 @@ class TestValidator(unittest.TestCase):
         logging.basicConfig(filename="/dev/null", level=logging.DEBUG)
 
     @patch("amd_debug.validator.SleepDatabase")
-    def setUp(self, _db_mock):
+    @patch("subprocess.run")
+    def setUp(self, _db_mock, _mock_run):
         """Set up a mock context for testing"""
         self.validator = SleepValidator(tool_debug=True, bios_debug=False)
 
@@ -157,9 +159,8 @@ class TestValidator(unittest.TestCase):
         self.validator.capture_wake_sources()
 
         # Validate debug messages
-        mock_record_debug.assert_any_call("Possible wakeup sources:")
         mock_record_debug.assert_any_call(
-            "└─ [/sys/devices/pci0000:00/0000:00:14.0]: enabled"
+            "Wakeup Source|Linux Device|Status\n|/sys/devices/pci0000:00/0000:00:14.0|enabled\n"
         )
 
         # Stop patches
@@ -694,7 +695,7 @@ class TestValidator(unittest.TestCase):
         # Test case 3: Randomized test
         mock_randint.side_effect = [7, 3]  # Random duration and wait
         self.validator.run(duration=10, count=1, wait=5, rand=True, logind=False)
-        mock_randint.assert_any_call(1, 10)
+        mock_randint.assert_any_call(4, 10)
         mock_randint.assert_any_call(1, 5)
         mock_run_countdown.assert_any_call("Suspending system", math.ceil(3 / 2))
         mock_run_countdown.assert_any_call("Collecting data", math.ceil(3 / 2))
@@ -705,19 +706,189 @@ class TestValidator(unittest.TestCase):
         mock_report_cycle.assert_called()
         mock_unlock_session.assert_called()
 
-        # Test case 4: Multiple cycles
+        # Test case 4: Randomized test, but too short of a duration
+        result = self.validator.run(
+            duration=4, count=1, wait=5, rand=True, logind=False
+        )
+        self.assertFalse(result)
+        mock_report_cycle.assert_called()
+
+        # Test case 5: Multiple cycles
         self.validator.run(duration=10, count=2, wait=5, rand=False, logind=False)
         self.assertEqual(mock_prep.call_count, 4)  # Includes previous calls
         self.assertEqual(mock_program_wakealarm.call_count, 4)
         self.assertEqual(mock_suspend_system.call_count, 4)
         self.assertEqual(mock_post.call_count, 4)
-        self.assertEqual(mock_report_cycle.call_count, 4)
+        self.assertEqual(mock_report_cycle.call_count, 5)
         self.assertEqual(mock_unlock_session.call_count, 3)
 
-        # Test case 5: suspend_system fails
+        # Test case 6: suspend_system fails
         mock_suspend_system.return_value = False
         result = self.validator.run(
             duration=10, count=1, wait=5, rand=False, logind=False
         )
         self.assertFalse(result)
         mock_report_cycle.assert_called()
+
+    @patch("os.path.exists")
+    @patch("builtins.open", new_callable=mock_open, read_data="3")
+    @patch("os.write")
+    @patch("os.open")
+    @patch("os.close")
+    def test_suspend_system_sysfs_success(
+        self,
+        mock_os_close,
+        mock_os_open,
+        mock_os_write,
+        _mock_open_file,
+        mock_path_exists,
+    ):
+        """Test suspend_system method using sysfs interface with success"""
+        # Mock wakeup_count file existence
+        mock_path_exists.side_effect = lambda path: "wakeup_count" in path
+
+        # Mock os.open and os.write
+        mock_os_open.return_value = 3
+        mock_os_write.return_value = None
+
+        # Call the method
+        result = self.validator.suspend_system()
+
+        # Assert the method returned True
+        self.assertTrue(result)
+
+        # Assert os.open and os.write were called
+        mock_os_open.assert_called_once_with(
+            "/sys/power/state", os.O_WRONLY | os.O_SYNC
+        )
+        mock_os_write.assert_called_once_with(3, b"mem")
+        mock_os_close.assert_called_once_with(3)
+
+    @patch("os.path.exists")
+    @patch("builtins.open", new_callable=mock_open, read_data="3")
+    @patch("os.write")
+    @patch("os.open")
+    @patch("os.close")
+    def test_suspend_system_sysfs_failure(
+        self,
+        mock_os_close,
+        mock_os_open,
+        mock_os_write,
+        _mock_open_file,
+        mock_path_exists,
+    ):
+        """Test suspend_system method using sysfs interface with failure"""
+        # Mock wakeup_count file existence
+        mock_path_exists.side_effect = lambda path: "wakeup_count" in path
+
+        # Mock os.open to raise OSError
+        mock_os_open.return_value = 3
+        mock_os_write.side_effect = OSError("Failed to write to state")
+
+        # Call the method
+        result = self.validator.suspend_system()
+
+        # Assert the method returned False
+        self.assertFalse(result)
+
+        # Assert os.open and os.write were called
+        mock_os_open.assert_called_once_with(
+            "/sys/power/state", os.O_WRONLY | os.O_SYNC
+        )
+        mock_os_write.assert_called_once_with(3, b"mem")
+        mock_os_close.assert_called_once_with(3)
+
+    @patch("os.path.exists")
+    @patch("builtins.open", new_callable=mock_open)
+    @patch("os.write")
+    @patch("os.open")
+    @patch("os.close")
+    def test_suspend_system_sysfs_no_wakeup_count(
+        self,
+        mock_os_close,
+        mock_os_open,
+        mock_os_write,
+        _mock_open_file,
+        mock_path_exists,
+    ):
+        """Test suspend_system method using sysfs interface with no wakeup_count file"""
+        # Mock wakeup_count file does not exist
+        mock_path_exists.return_value = False
+
+        # Mock os.open and os.write
+        mock_os_open.return_value = 3
+        mock_os_write.return_value = None
+
+        # Call the method
+        result = self.validator.suspend_system()
+
+        # Assert the method returned True
+        self.assertTrue(result)
+
+        # Assert os.open and os.write were called
+        mock_os_open.assert_called_once_with(
+            "/sys/power/state", os.O_WRONLY | os.O_SYNC
+        )
+        mock_os_write.assert_called_once_with(3, b"mem")
+        mock_os_close.assert_called_once_with(3)
+
+    @patch("os.path.exists")
+    @patch("os.open")
+    @patch("os.write")
+    @patch("os.close")
+    def test_toggle_nvidia_file_not_exists(
+        self, mock_close, mock_write, mock_open, mock_exists
+    ):
+        """Test toggle_nvidia returns True if NVIDIA suspend file does not exist"""
+        mock_exists.return_value = False
+        result = self.validator.toggle_nvidia(b"suspend")
+        self.assertTrue(result)
+        mock_open.assert_not_called()
+        mock_write.assert_not_called()
+        mock_close.assert_not_called()
+
+    @patch("os.path.exists")
+    @patch("os.open")
+    @patch("os.write")
+    @patch("os.close")
+    def test_toggle_nvidia_success(
+        self, mock_close, mock_write, mock_open, mock_exists
+    ):
+        """Test toggle_nvidia writes value and returns True on success"""
+        mock_exists.return_value = True
+        mock_open.return_value = 42
+        mock_write.return_value = None
+        with patch.object(self.validator.db, "record_debug") as mock_record_debug:
+            result = self.validator.toggle_nvidia(b"suspend")
+            self.assertTrue(result)
+            mock_open.assert_called_once_with(
+                "/proc/driver/nvidia/suspend", os.O_WRONLY | os.O_SYNC
+            )
+            mock_write.assert_called_once_with(42, b"suspend")
+            mock_close.assert_called_once_with(42)
+            mock_record_debug.assert_called_once_with(
+                "Wrote b'suspend' to NVIDIA driver"
+            )
+
+    @patch("os.path.exists")
+    @patch("os.open")
+    @patch("os.write")
+    @patch("os.close")
+    def test_toggle_nvidia_oserror(
+        self, mock_close, mock_write, mock_open, mock_exists
+    ):
+        """Test toggle_nvidia handles OSError and returns False"""
+        mock_exists.return_value = True
+        mock_open.return_value = 99
+        mock_write.side_effect = OSError("write error")
+        with patch.object(
+            self.validator.db, "record_cycle_data"
+        ) as mock_record_cycle_data:
+            result = self.validator.toggle_nvidia(b"resume")
+            self.assertFalse(result)
+            mock_open.assert_called_once_with(
+                "/proc/driver/nvidia/suspend", os.O_WRONLY | os.O_SYNC
+            )
+            mock_write.assert_called_once_with(99, b"resume")
+            mock_close.assert_called_once_with(99)
+            mock_record_cycle_data.assert_called_once()
